import { world, EquipmentSlot } from "@minecraft/server";

world.afterEvents.entityDie.subscribe((data) => {
    const { deadEntity } = data;

    const nearbyPlayers = deadEntity.dimension.getPlayers({
        location: deadEntity.location,
        maxDistance: 5
    });

    if (nearbyPlayers.length === 0) return;

    const player = nearbyPlayers[0];
    const equippable = player.getComponent("minecraft:equippable");
    const slot = equippable?.getEquipmentSlot(EquipmentSlot.Mainhand);
    const item = slot?.getItem();

    if (!item || !item.typeId.startsWith("tunekeshi:b")) return;

    let killCount = 0;
    let proudSoul = 0;
    const lore = item.getLore();
    for (const line of lore) {
        if (line.startsWith("§r§4KillCount : ")) {
            const num = Number(line.slice("§r§4KillCount : ".length));
            if (!isNaN(num)) killCount = num;
        }
        if (line.startsWith("§r§5ProudSoul : ")) {
            const num = Number(line.slice("§r§5ProudSoul : ".length));
            if (!isNaN(num)) proudSoul = num;
        }
    }
    killCount += 1;
    const familyComp = deadEntity.getComponent("minecraft:type_family");
    const families = familyComp?.families ?? [];
    const isMonster = families.includes("monster");
    const increase = isMonster
        ? Math.floor(Math.random() * 5) + 1  // 1〜5
        : Math.floor(Math.random() * 3) + 1; // 1〜3
    proudSoul += increase;
    const newLore = [
        `§r§4KillCount : ${killCount}`,
        `§r§5ProudSoul : ${proudSoul}`
    ];
    item.setLore(newLore);
    slot.setItem(undefined);
    slot.setItem(item);
});