import { world, system, EquipmentSlot } from "@minecraft/server";

const saCosts = {
  "tunekeshi:b3": 10,
  "tunekeshi:b4": 10,
  "tunekeshi:b6": 10,
  "tunekeshi:b7": 10,
  "tunekeshi:b8": 10,
  "tunekeshi:b9": 10,
  "tunekeshi:b10": 10,
  "tunekeshi:b11": 10,
  "tunekeshi:b12": 10,
  "tunekeshi:b14": 10,
  "tunekeshi:b16": 10,
  "tunekeshi:b17": 10,
  "tunekeshi:b18": 10,
  "tunekeshi:b19": 10,
  "tunekeshi:b20": 10,
  "tunekeshi:b21": 10,
  "tunekeshi:b22": 10,
  "tunekeshi:b25": 10,
  "tunekeshi:b26": 10,
  "tunekeshi:b27": 10,
  "tunekeshi:b28": 10,
  "tunekeshi:b29": 10,
  "tunekeshi:b30": 10,
  "tunekeshi:b31": 10,
  "tunekeshi:b32": 10,
  "tunekeshi:b33": 10,
};

world.beforeEvents.itemUse.subscribe(event => {
  const { itemStack, source } = event;
  if (!itemStack || !source || !itemStack.typeId.startsWith("tunekeshi:b")) return;

  const cost = saCosts[itemStack.typeId];
  if (cost === undefined) return;

  const equippable = source.getComponent("minecraft:equippable");
  const item = equippable?.getEquipment(EquipmentSlot.Mainhand);
  if (!item) return;

  const lore = item.getLore();
  let proudSoul = 0;

  for (const line of lore) {
    if (line.startsWith("§r§5ProudSoul : ")) {
      proudSoul = Number(line.slice(16));
      break;
    }
  }

  if (isNaN(proudSoul) || proudSoul < cost) return;

  const newLore = lore.map(line =>
    line.startsWith("§r§5ProudSoul : ")
      ? `§r§5ProudSoul : ${Math.max(proudSoul - cost, 0)}`
      : line
  );

  system.run(() => {
    item.setLore(newLore);
    equippable.getEquipmentSlot(EquipmentSlot.Mainhand).setItem(item);

    executeItemAction(source, itemStack.typeId);
  });
});

function executeItemAction(source, itemId) {
  const head = source.getHeadLocation();
  const view = source.getViewDirection();

  switch (itemId) {
    case "tunekeshi:b3":
      source.runCommandAsync("summon tunekeshi:sa_b3 ^ ^1.8 ^1.5");
      break;
    case "tunekeshi:b4":
      spawnProjectile(source, "tunekeshi:sa_b4", head, view);
      break;
    case "tunekeshi:b6":
      source.runCommandAsync("execute as @e[r=3,type=!player] at @s run summon lightning_bolt");
      source.runCommandAsync("execute as @e[r=3,type=!player] at @s run summon tunekeshi:ice");
      break;
    case "tunekeshi:b7":
      source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^3.8 ^1.5");
      source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^2.8 ^1.5");
      source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^1.8 ^1.5");
      source.runCommandAsync("effect @s resistance 1 255 true");
      source.runCommandAsync("particle sa_b7 ^ ^2.8 ^2.5");
      break;
    case "tunekeshi:b8":
      source.runCommandAsync("summon tunekeshi:sa_b8");
      break;
    case "tunekeshi:b9":
      source.addTag("wither");
      break;
    case "tunekeshi:b10":
      source.runCommandAsync("execute as @e[r=10,type=!player] at @s run summon lightning_bolt");
      source.runCommandAsync("effect @s fire_resistance 1 255 true");
      break;
    case "tunekeshi:b11":
      source.runCommandAsync("execute as @e[r=10,type=!tunekeshi:ice,type=!player] at @s run summon tunekeshi:ice");
      break;
    case "tunekeshi:b12":
      spawnProjectile(source, "tunekeshi:sa_b12", head, view);
      break;
    case "tunekeshi:b14":
      source.runCommandAsync("scoreboard players add @s level1 7");
      break;
    case "tunekeshi:b16":
      source.runCommandAsync("summon tunekeshi:sa_b16 ^ ^2.8 ^1.5");
      break;
    case "tunekeshi:b17":
      spawnProjectile(source, "tunekeshi:sa_b17", head, view);
      break;
    case "tunekeshi:b18":
      spawnProjectile(source, "tunekeshi:sa_b18", head, view);
      break;
    case "tunekeshi:b19":
      source.runCommandAsync("summon tunekeshi:sa_b19 ^ ^2.8 ^1.5");
      break;
    case "tunekeshi:b20":
      source.runCommandAsync("summon tunekeshi:sa_b19 ^ ^2.8 ^1.5");
      break;
    case "tunekeshi:b21":
      spawnProjectile(source, "tunekeshi:sa_b21", head, view);
      break;
    case "tunekeshi:b22":
      spawnProjectile(source, "tunekeshi:sa_b22", head, view);
      break;
    case "tunekeshi:b25":
    case "tunekeshi:b26":
    case "tunekeshi:b29":
    case "tunekeshi:b30":
    case "tunekeshi:b31":
    case "tunekeshi:b32":
    case "tunekeshi:b33":
      spawnProjectile(source, "tunekeshi:sa_b18", head, view);
      break;
    case "tunekeshi:b27":
      applyKnock(source, view, 3);
      source.runCommandAsync("execute at @s run particle minecraft:huge_explosion_emitter ^^^-1");
      source.runCommandAsync("execute at @s run damage @e[r=4,type=!player,type=!item] 10");
      source.runCommandAsync("execute at @s run particle sa_b27 ^^^");
      break;
    case "tunekeshi:b28":
      applyKnock(source, view, 3);
      source.runCommandAsync("execute at @s run particle minecraft:huge_explosion_emitter ^^^-1");
      source.runCommandAsync("execute at @s run damage @e[r=4,type=!player,type=!item] 7");
      source.runCommandAsync("execute at @s run particle sa_b28 ^^^");
      source.runCommandAsync("execute as @e[r=4,type=!player,type=!tunekeshi:ice] at @s run summon tunekeshi:ice");
      break;
  }
}

function spawnProjectile(source, projectileId, head, view) {
  const spawnLocation = {
    x: head.x + view.x,
    y: head.y + view.y,
    z: head.z + view.z
  };
  const projectile = source.dimension.spawnEntity(projectileId, spawnLocation);
  projectile.applyImpulse({
    x: view.x * 3,
    y: view.y * 3,
    z: view.z * 3
  });
}

function applyKnock(source, view, strength) {
  const knockX = view.x;
  const knockY = view.y * 0.1;
  const knockZ = view.z;
  source.applyKnockback(knockX, knockZ, strength, knockY);
}